#! /bin/sh
# Install script for privoxy by Martijn Dekker <martijn@inlv.org>
# Generated by privoxy.SlackBuild for privoxy-3.0.8p1-i486-1McD.tgz on 26-Jan-2008

function upd_conf {
 	# If there's no config file by that name, mv it over:
 	if ! test -e $1; then
 		mv $1.new $1
 	elif diff -q $1 $1.new; then	# toss the redundant copy
 		rm $1.new
 	elif ! test -s $1.new; then
 		rm $1.new   # do not keep empty initial files (e.g. log)
 	fi
 	# Otherwise, we leave the .new copy for the admin to consider...
}

function free_user_id {
 	# Find a free user-ID >= 100 (should be < 1000 so it's not a normal user)
 	local FREE_USER_ID=100
 	while grep -q "^.*:.*:$FREE_USER_ID:.*:.*:.*:" etc/passwd; do
 		let FREE_USER_ID++
 	done
 	echo $FREE_USER_ID
}

# Figure out our root directory
ROOTDIR=$(pwd)
unset CHROOT
if test "$ROOTDIR" != "/"; then
 	CHROOT="chroot $ROOTDIR "
 	ROOTDIR="${ROOTDIR}/"
fi

TMPFILE=$(mktemp /tmp/privoxy.XXXXXX)

# In case we are upgrading to a new version, update path to the manual's
# directory in existing config file. We do this before updating the config
# files; it's not worth leaving a .new file over, and we want the manual to
# always keep working even if the admin never notices the .new file.
if test -f etc/privoxy/config; then
 	sed "s@/usr/doc/privoxy-.*/user-manual@/usr/doc/privoxy-3.0.8-stable/user-manual@"	\
 		etc/privoxy/config > $TMPFILE
 	if ! diff -q etc/privoxy/config $TMPFILE > /dev/null; then
 		cat $TMPFILE > etc/privoxy/config	# this doesn't alter perms
 	fi
fi

# Edit rc.privoxy script and logrotate file with currently used user/group
# (it's not worth leaving a .new file over)
if test -f etc/rc.d/rc.privoxy; then
 	sed '	/^PRIVOXY_USER=/	s/=.*/="privoxy"/
 		/^PRIVOXY_GROUP=/	s/=.*/="privoxy"/
 		' etc/rc.d/rc.privoxy > $TMPFILE
 	if ! diff etc/rc.d/rc.privoxy $TMPFILE > /dev/null; then
 		printf 'Changing user/group in %setc/rc.d/rc.privoxy to ' $ROOTDIR 1>&2
 		printf '\033[1mprivoxy\033[0m:\033[1mprivoxy\033[0m.\n' 1>&2
 		cat $TMPFILE > etc/rc.d/rc.privoxy	# this doesn't alter perms
 	fi
fi
if test -f etc/logrotate.d/privoxy; then
 	sed '/create/	s/\([0-9][0-9][0-9]\)[[:blank:]].*/\1 privoxy privoxy/
 		' etc/logrotate.d/privoxy > $TMPFILE
 	if ! diff etc/logrotate.d/privoxy $TMPFILE > /dev/null; then
 		printf 'Changing user/group in %setc/logrotate.d/privoxy to ' $ROOTDIR 1>&2
 		printf '\033[1mprivoxy\033[0m:\033[1mprivoxy\033[0m.\n' 1>&2
 		cat $TMPFILE > etc/logrotate.d/privoxy	# this doesn't alter perms
 	fi
fi

rm $TMPFILE

# Set up group if needed
grep --quiet '^privoxy:' etc/group || $CHROOT /usr/sbin/groupadd privoxy

# Set up user: add it if it doesn't exist, update it if it already does.
# In any case, make sure it has or gets a user ID between 100 and 999.
# If username changed, determine old username so we can change it (look
# for entry with homedir /var/{log,run}/privoxy).
if OLD_ENTRY=$(grep -m 1 '^privoxy:' etc/passwd) \
 	|| OLD_ENTRY=$(grep -m 1 \
 		':/var/[lr][ou][gn]/privoxy:[a-z/]*$' etc/passwd)
then
 	# Modify existing user
 	OLD_USER=$(echo $OLD_ENTRY | cut -f 1 -d ':')
 	OLD_USER_ID=$(echo $OLD_ENTRY | cut -f 3 -d ':')
 	OLD_GROUP_ID=$(echo $OLD_ENTRY | cut -f 4 -d ':')
 	if test "$OLD_USER" != "privoxy"; then
 		printf '\033[31mChanging\033[0m unprivileged user \033[1m%s\033[0m to' $OLD_USER 1>&2
 	else
 		printf 'Updating unprivileged user' 1>&2
 	fi
 	printf ' \033[1mprivoxy\033[0m with group \033[1mprivoxy\033[0m.\n' 1>&2
 	if test $OLD_USER_ID -ge 1000; then
 		USER_ID=$(free_user_id)
 		find etc/privoxy -user $OLD_USER_ID |
 			xargs -r chown $USER_ID
 	else
 		USER_ID=$OLD_USER_ID
 	fi
 	find etc/privoxy -group $OLD_GROUP_ID |
 		xargs -r chgrp privoxy
 	$CHROOT /usr/sbin/usermod \
 		-l privoxy \
 		-g privoxy \
 		-d /var/run/privoxy \
 		-u $USER_ID \
 		-s /bin/false \
 		-c "Privacy Proxy" \
 		$OLD_USER
else
 	# Add new user
 	printf 'Adding unprivileged user \033[1mprivoxy\033[0m with group \033[1mprivoxy\033[0m.\n' 1>&2
 	$CHROOT /usr/sbin/useradd \
 		-g privoxy \
 		-d /var/run/privoxy \
 		-u $(free_user_id) \
 		-s /bin/false \
 		-c "Privacy Proxy" \
 		privoxy
fi

# Set up permissions
$CHROOT /bin/chown -R privoxy:privoxy \
	/etc/privoxy/config.new \
	/etc/privoxy/trust.new \
	/etc/privoxy/global.action.new \
	/etc/privoxy/standard.action.new \
	/etc/privoxy/user.action.new \
	/etc/privoxy/default.filter.new \
	/etc/privoxy/user.filter.new  \
 	/var/log/privoxy \
 	/var/run/privoxy 
$CHROOT /bin/chgrp privoxy /etc/privoxy/default.action.new

# Update configuration files
upd_conf ${ROOTDIR}etc/logrotate.d/privoxy
upd_conf ${ROOTDIR}etc/privoxy/config
upd_conf ${ROOTDIR}etc/privoxy/default.action
upd_conf ${ROOTDIR}etc/privoxy/default.filter
upd_conf ${ROOTDIR}etc/privoxy/global.action
upd_conf ${ROOTDIR}etc/privoxy/standard.action
upd_conf ${ROOTDIR}etc/privoxy/templates/blocked
upd_conf ${ROOTDIR}etc/privoxy/templates/cgi-error-404
upd_conf ${ROOTDIR}etc/privoxy/templates/cgi-error-bad-param
upd_conf ${ROOTDIR}etc/privoxy/templates/cgi-error-disabled
upd_conf ${ROOTDIR}etc/privoxy/templates/cgi-error-file
upd_conf ${ROOTDIR}etc/privoxy/templates/cgi-error-file-read-only
upd_conf ${ROOTDIR}etc/privoxy/templates/cgi-error-modified
upd_conf ${ROOTDIR}etc/privoxy/templates/cgi-error-parse
upd_conf ${ROOTDIR}etc/privoxy/templates/cgi-style.css
upd_conf ${ROOTDIR}etc/privoxy/templates/connect-failed
upd_conf ${ROOTDIR}etc/privoxy/templates/default
upd_conf ${ROOTDIR}etc/privoxy/templates/edit-actions-add-url-form
upd_conf ${ROOTDIR}etc/privoxy/templates/edit-actions-for-url
upd_conf ${ROOTDIR}etc/privoxy/templates/edit-actions-for-url-filter
upd_conf ${ROOTDIR}etc/privoxy/templates/edit-actions-list
upd_conf ${ROOTDIR}etc/privoxy/templates/edit-actions-list-button
upd_conf ${ROOTDIR}etc/privoxy/templates/edit-actions-list-section
upd_conf ${ROOTDIR}etc/privoxy/templates/edit-actions-list-url
upd_conf ${ROOTDIR}etc/privoxy/templates/edit-actions-remove-url-form
upd_conf ${ROOTDIR}etc/privoxy/templates/edit-actions-url-form
upd_conf ${ROOTDIR}etc/privoxy/templates/forwarding-failed
upd_conf ${ROOTDIR}etc/privoxy/templates/mod-local-help
upd_conf ${ROOTDIR}etc/privoxy/templates/mod-support-and-service
upd_conf ${ROOTDIR}etc/privoxy/templates/mod-title
upd_conf ${ROOTDIR}etc/privoxy/templates/mod-unstable-warning
upd_conf ${ROOTDIR}etc/privoxy/templates/no-such-domain
upd_conf ${ROOTDIR}etc/privoxy/templates/show-request
upd_conf ${ROOTDIR}etc/privoxy/templates/show-status
upd_conf ${ROOTDIR}etc/privoxy/templates/show-status-file
upd_conf ${ROOTDIR}etc/privoxy/templates/show-url-info
upd_conf ${ROOTDIR}etc/privoxy/templates/show-version
upd_conf ${ROOTDIR}etc/privoxy/templates/toggle
upd_conf ${ROOTDIR}etc/privoxy/templates/toggle-mini
upd_conf ${ROOTDIR}etc/privoxy/templates/untrusted
upd_conf ${ROOTDIR}etc/privoxy/trust
upd_conf ${ROOTDIR}etc/privoxy/user.action
upd_conf ${ROOTDIR}etc/privoxy/user.filter
upd_conf ${ROOTDIR}etc/rc.d/rc.privoxy

# Initialize /var files (e.g. logs) if not already present
upd_conf var/log/privoxy/jarfile > /dev/null 2>&1
upd_conf var/log/privoxy/logfile > /dev/null 2>&1

# Make us start on boot
if ! grep -q "rc.privoxy start" etc/rc.d/rc.local; then
 	echo "Adding Privoxy to ${ROOTDIR}etc/rc.d/rc.local so it starts at bootup time." 1>&2
 	{	echo ''
 		echo '# Privoxy, the privacy web proxy'
 		echo 'test -x /etc/rc.d/rc.privoxy && /etc/rc.d/rc.privoxy start'
 	} >> etc/rc.d/rc.local
else
 	echo "Privoxy is already in ${ROOTDIR}etc/rc.d/rc.local ."
fi

# Make us stop on shutdown
if test ! -e etc/rc.d/rc.local_shutdown; then
 	touch etc/rc.d/rc.local_shutdown
 	chmod +x etc/rc.d/rc.local_shutdown
fi
if ! grep -q "rc.privoxy stop" etc/rc.d/rc.local_shutdown 2>/dev/null; then
 	echo "Adding Privoxy to ${ROOTDIR}etc/rc.d/rc.local_shutdown so it stops at shutdown time." 1>&2
 	{	echo ''
 		echo '# Privoxy, the privacy web proxy'
 		echo 'test -x /etc/rc.d/rc.privoxy && /etc/rc.d/rc.privoxy stop'
 	} >> etc/rc.d/rc.local_shutdown
else
 	echo "Privoxy is already in ${ROOTDIR}etc/rc.d/rc.local_shutdown ."
fi

